"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.compileWithClangPlus = exports.compileWithClang = exports.compileWithGPlus = exports.compileWithGcc = exports.compileWith = void 0;
const child_process_1 = __importDefault(require("child_process"));
const compilationFailed_1 = __importDefault(require("./errors/compilationFailed"));
/**
 * Compiles C/C++ source.
 * @category Compilation
 * @param {string} compilerName Compiler to use.
 * @param {string} inputFile Source file to compile.
 * @param {string} outputFile Output file name.
 * @param {boolean} link Should result be linked.
 * @returns {Promise<{ inputFile: string }>}
 * @throws {@link CompilationFailedError} if compilation fails.
 */
function compileWith(compilerName, inputFile, outputFile, link) {
    const command = `${compilerName} ${inputFile} -o ${outputFile}${link ? '' : ' -c'}`;
    return new Promise((resolve, reject) => {
        child_process_1.default.exec(command, (error) => {
            if (error) {
                reject(new compilationFailed_1.default(compilerName, inputFile));
            }
            resolve({ inputFile });
        });
    });
}
exports.compileWith = compileWith;
/**
 * Compiles C/C++ source using gcc.
 * @category Compilation
 * @param {string} inputFile Source file to compile.
 * @param {string} outputFile Output file name.
 * @param {boolean} link Should result be linked.
 * @returns {Promise<{ inputFile: string }>}
 * @throws {@link CompilationFailedError} if compilation fails.
 */
function compileWithGcc(inputFile, outputFile, link = true) {
    return compileWith('gcc', inputFile, outputFile, link);
}
exports.compileWithGcc = compileWithGcc;
/**
 * Compiles C/C++ source using g++.
 * @category Compilation
 * @param {string} inputFile Source file to compile.
 * @param {string} outputFile Output file name.
 * @param {boolean} link Should result be linked.
 * @returns {Promise<{ inputFile: string }>}
 * @throws {@link CompilationFailedError} if compilation fails.
 */
function compileWithGPlus(inputFile, outputFile, link = true) {
    return compileWith('g++', inputFile, outputFile, link);
}
exports.compileWithGPlus = compileWithGPlus;
/**
 * Compiles C/C++ source using clang.
 * @category Compilation
 * @param {string} inputFile Source file to compile.
 * @param {string} outputFile Output file name.
 * @param {boolean} link Should result be linked.
 * @returns {Promise<{ inputFile: string }>}
 * @throws {@link CompilationFailedError} if compilation fails.
 */
function compileWithClang(inputFile, outputFile, link = true) {
    return compileWith('clang', inputFile, outputFile, link);
}
exports.compileWithClang = compileWithClang;
/**
 * Compiles C/C++ source using clang++.
 * @category Compilation
 * @param {string} inputFile Source file to compile.
 * @param {string} outputFile Output file name.
 * @param {boolean} link Should result be linked.
 * @returns {Promise<{ inputFile: string }>}
 * @throws {@link CompilationFailedError} if compilation fails.
 */
function compileWithClangPlus(inputFile, outputFile, link = true) {
    return compileWith('clang++', inputFile, outputFile, link);
}
exports.compileWithClangPlus = compileWithClangPlus;
//# sourceMappingURL=compilation.js.map