"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkForClangPlus = exports.checkForClang = exports.checkForGPlus = exports.checkForGcc = exports.checkForCompiler = void 0;
const child_process_1 = __importDefault(require("child_process"));
const compilerNotFound_1 = __importDefault(require("./errors/compilerNotFound"));
const isVersionString_1 = __importDefault(require("./utils/isVersionString"));
/**
 * Checks whether a compiler is in the system path (i.e. PATH).
 * @category Compiler Detection
 * @param {string} compilerName The compiler to look for.
 * @returns {Promise<{compiler: string, version: string}>}
 * @throws {@link CompilerNotFoundError} if specified compiler can't be found.
 */
function checkForCompiler(compilerName) {
    return new Promise((resolve, reject) => {
        child_process_1.default.exec(`${compilerName} --version`, (error, stdout) => {
            if (error) {
                reject(new compilerNotFound_1.default(compilerName));
            }
            const outputLines = stdout.replace(/\r/g, '').trim().split('\n');
            const splitFirstLine = outputLines[0].split(' ');
            const version = splitFirstLine[splitFirstLine.length - 1].split('-')[0];
            if ((0, isVersionString_1.default)(version)) {
                resolve({ compiler: compilerName, version });
            }
            reject(new compilerNotFound_1.default(compilerName));
        });
    });
}
exports.checkForCompiler = checkForCompiler;
/**
 * Checks whether gcc is in the system path (i.e. PATH).
 * @category Compiler Detection
 * @returns {Promise<{compiler: string, version: string}>}
 * @throws {@link CompilerNotFoundError} if gcc can't be found in the system path.
 */
function checkForGcc() {
    return checkForCompiler('gcc');
}
exports.checkForGcc = checkForGcc;
/**
 * Checks whether g++ is in the system path (i.e. PATH).
 * @category Compiler Detection
 * @returns {Promise<{compiler: string, version: string}>}
 * @throws {@link CompilerNotFoundError} if g++ can't be found in the system path.
 */
function checkForGPlus() {
    return checkForCompiler('g++');
}
exports.checkForGPlus = checkForGPlus;
/**
 * Checks whether clang is in the system path (i.e. PATH).
 * @category Compiler Detection
 * @returns {Promise<{compiler: string, version: string}>}
 * @throws {@link CompilerNotFoundError} if clang can't be found in the system path.
 */
function checkForClang() {
    return checkForCompiler('clang');
}
exports.checkForClang = checkForClang;
/**
 * Checks whether clang++ is in the system path (i.e. PATH).
 * @category Compiler Detection
 * @returns {Promise<{compiler: string, version: string}>}
 * @throws {@link CompilerNotFoundError} if clang++ can't be found in the system path.
 */
function checkForClangPlus() {
    return checkForCompiler('clang++');
}
exports.checkForClangPlus = checkForClangPlus;
//# sourceMappingURL=compilerDetection.js.map